<?php

namespace Inventory\Pricelist\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Inventory\Pricelist\Http\Requests\PricelistRequest;
use Inventory\Pricelist\Interfaces\PricelistRepositoryInterface;
use Inventory\Pricelist\Models\Pricelist;

/**
 * Resource controller class for pricelist.
 */
class PricelistResourceController extends BaseController
{

    /**
     * Initialize pricelist resource controller.
     *
     * @param type PricelistRepositoryInterface $pricelist
     *
     * @return null
     */
    public function __construct(PricelistRepositoryInterface $pricelist)
    {
        parent::__construct();
        $this->repository = $pricelist;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Inventory\Pricelist\Repositories\Criteria\PricelistResourceCriteria::class);
    }

    /**
     * Display a list of pricelist.
     *
     * @return Response
     */
    public function index(PricelistRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Inventory\Pricelist\Repositories\Presenter\PricelistPresenter::class)
                ->$function();
        }

        $pricelists = $this->repository->paginate();

        return $this->response->setMetaTitle(trans('pricelist::pricelist.names'))
            ->view('pricelist::pricelist.index', true)
            ->data(compact('pricelists', 'view'))
            ->output();
    }

    /**
     * Display pricelist.
     *
     * @param Request $request
     * @param Model   $pricelist
     *
     * @return Response
     */
    public function show(PricelistRequest $request, Pricelist $pricelist)
    {

        if ($pricelist->exists) {
            $view = 'pricelist::pricelist.show';
        } else {
            $view = 'pricelist::pricelist.new';
        }

        return $this->response->setMetaTitle(trans('app.view') . ' ' . trans('pricelist::pricelist.name'))
            ->data(compact('pricelist'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new pricelist.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(PricelistRequest $request)
    {

        $pricelist = $this->repository->newInstance([]);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('pricelist::pricelist.name')) 
            ->view('pricelist::pricelist.create', true) 
            ->data(compact('pricelist'))
            ->output();
    }

    /**
     * Create new pricelist.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(PricelistRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $pricelist                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('pricelist::pricelist.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('pricelist/pricelist/' . $pricelist->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/pricelist/pricelist'))
                ->redirect();
        }

    }

    /**
     * Show pricelist for editing.
     *
     * @param Request $request
     * @param Model   $pricelist
     *
     * @return Response
     */
    public function edit(PricelistRequest $request, Pricelist $pricelist)
    {
        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('pricelist::pricelist.name'))
            ->view('pricelist::pricelist.edit', true)
            ->data(compact('pricelist'))
            ->output();
    }

    /**
     * Update the pricelist.
     *
     * @param Request $request
     * @param Model   $pricelist
     *
     * @return Response
     */
    public function update(PricelistRequest $request, Pricelist $pricelist)
    {
        try {
            $attributes = $request->all();

            $pricelist->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('pricelist::pricelist.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('pricelist/pricelist/' . $pricelist->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('pricelist/pricelist/' . $pricelist->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the pricelist.
     *
     * @param Model   $pricelist
     *
     * @return Response
     */
    public function destroy(PricelistRequest $request, Pricelist $pricelist)
    {
        try {

            $pricelist->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('pricelist::pricelist.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('pricelist/pricelist/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('pricelist/pricelist/' . $pricelist->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple pricelist.
     *
     * @param Model   $pricelist
     *
     * @return Response
     */
    public function delete(PricelistRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('pricelist::pricelist.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('pricelist/pricelist'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/pricelist/pricelist'))
                ->redirect();
        }

    }

    /**
     * Restore deleted pricelists.
     *
     * @param Model   $pricelist
     *
     * @return Response
     */
    public function restore(PricelistRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('pricelist::pricelist.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/pricelist/pricelist'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/pricelist/pricelist/'))
                ->redirect();
        }

    }

}
